@@ -13,12 +13,17 @@
 #include <ctype.h>	 /* for tolower(), toupper() */
 #include <math.h>
 
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
+
+#if !defined(XC_WIN32) || defined(TCL_WRAPPER)
 #define  XK_MISCELLANY
 #define  XK_LATIN1
 #define  XK_XKB_KEYS
 #include <X11/keysymdef.h>
+#endif
 
 /*----------------------------------------------------------------------*/
 /* Local includes							*/
@@ -164,6 +169,24 @@ int boundfunction(xcWidget window, int keywstate, short *retnum)
    return tempfunc;
 }
 
+/*--------------------------------------------------------------*/
+/* Check if an entry exists for a given key-function pair	*/
+/*--------------------------------------------------------------*/
+
+int isbound(xcWidget window, int keywstate, int function, short value)
+{
+   keybinding *ksearch;
+
+   for (ksearch = keylist; ksearch != NULL; ksearch = ksearch->nextbinding)
+      if (keywstate == ksearch->keywstate && function == ksearch->function)
+	 if (window == ALL_WINDOWS || window == ksearch->window ||
+		ksearch->window == ALL_WINDOWS)
+	    if (value == -1 || value == ksearch->value || ksearch->value == -1)
+	       return TRUE;
+
+   return FALSE;
+}
+
 /*--------------------------------------------------------------*/
 /* Return the string associated with a function, or NULL if the	*/
 /* function value is out-of-bounds.				*/
@@ -323,7 +346,7 @@ char *function_binding_to_string(xcWidget window, int function)
 }
 
 /*--------------------------------------------------------------*/
-/* Convert a key into a string representin all of its function	*/
+/* Convert a key into a string representing all of its function	*/
 /* bindings.							*/
 /*--------------------------------------------------------------*/
 
@@ -356,6 +379,33 @@ char *key_binding_to_string(xcWidget window, int keywstate)
    return(retstr);
 }
 
+/*--------------------------------------------------------------*/
+/* Return an allocated string name of the function that		*/
+/* is bound to the indicated key state for the indicated	*/
+/* window and compattible with the current event mode.		*/
+/*--------------------------------------------------------------*/
+
+char *compat_key_to_string(xcWidget window, int keywstate)
+{
+   char *retstr, *tmpstr;
+   int function;
+
+   function = boundfunction(window, keywstate, NULL);
+   tmpstr = func_to_string(function);
+
+   /* Pass back "Nothing" for unbound key states, since a	*/
+   /* wrapper script may want to use the result as an action.	*/
+
+   if (tmpstr == NULL) {
+      retstr = (char *)malloc(8);
+      strcpy(retstr, "Nothing");
+   }
+   else
+      retstr = strdup(tmpstr);
+
+   return(retstr);
+}
+
 /*--------------------------------------------------------------*/
 /* Convert a key sym into a string				*/
 /*--------------------------------------------------------------*/
@@ -369,7 +419,11 @@ char *key_to_string(int keywstate)
 
    ks = keywstate & 0xffff;
    kmod = keywstate >> 16;
+#if defined(XC_WIN32) && defined(TCL_WRAPPER)
+   if (ks != NoSymbol) str = XKeysymToString_TkW32(ks);
+#else
    if (ks != NoSymbol) str = XKeysymToString(ks);
+#endif
 
    kptr = (char *)malloc(32);
    kptr[0] = '\0';
@@ -509,7 +563,7 @@ int add_vbinding(xcWidget window, int keywstate, int function, short value)
 
    /* If key is already bound to the function, ignore it */
 
-   if (boundfunction(window, keywstate, NULL) == function) return 1;
+   if (isbound(window, keywstate, function, value)) return 1;
 	   
    /* Add the new key binding */
 
@@ -761,8 +815,10 @@ static int button1mode = XCF_Wire;
 /* Re-bind BUTTON1 to the indicated function and optional value */
 /*--------------------------------------------------------------*/
 
-void mode_rebinding(xcWidget window, int newmode, int newvalue)
+void mode_rebinding(int newmode, int newvalue)
 {
+   xcWidget window = areawin->area;
+
    remove_binding(window, BUTTON1, button1mode);
    add_vbinding(window, BUTTON1, newmode, (short)newvalue);
    button1mode = newmode;
@@ -774,10 +830,11 @@ void mode_rebinding(xcWidget window, int newmode, int newvalue)
 /* mode, but return the keybinding to its previous state.	*/
 /*--------------------------------------------------------------*/
 
-void mode_tempbinding(xcWidget window, int newmode, int newvalue)
+void mode_tempbinding(int newmode, int newvalue)
 {
    short saveval;
    XPoint cpos;
+   xcWidget window = areawin->area;
 
    if (boundfunction(window, BUTTON1, &saveval) == button1mode) {
       remove_binding(window, BUTTON1, button1mode);
