@@ -10,10 +10,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifndef _MSC_VER
 #include <unistd.h>
+#endif
 #include <time.h>
+#ifndef _MSC_VER
 #include <sys/wait.h>   /* for waitpid() */
 #include <sys/time.h>
+#endif
 #include <sys/types.h>
 #include <signal.h>
 #include <fcntl.h>
@@ -29,8 +33,10 @@
 
 #include <tk.h>
 
+#ifndef _MSC_VER
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 
 #include "colordefs.h"
 #include "xcircuit.h"
@@ -83,16 +89,74 @@ int spice_state;	/* Track state of the simulator */
 /* Global variable definitions				*/
 /*------------------------------------------------------*/
 
-pid_t spiceproc = -1;	   /* ngspice process 			*/
-int   pipeRead, pipeWrite; /* I/O pipe pair for ngspice		*/
+#ifndef _MSC_VER
+pid_t spiceproc = -1;	   	/* ngspice process	*/
+#else
+HANDLE spiceproc = INVALID_HANDLE_VALUE;
+#endif
+
+int   pipeRead, pipeWrite; /* I/O pipe pair for ngspice	*/
 
 /*------------------------------------------------------*/
 /* Start a ngspice process and arrange the I/O pipes  	*/
 /* (Commented lines cause ngspice to relay its stderr 	*/
 /* to xcircuit's stderr)				*/
+/*							*/
+/* Return codes:					*/
+/*   0	success						*/
+/*  -1	failed---ngspice exited with error condition	*/
+/*  -2	exec failed---e.g., unable to find ngspice	*/
+/*   1	ngspice is already running			*/
 /*------------------------------------------------------*/
 
-void start_spice()
+#ifdef _MSC_VER
+
+int start_spice()
+{
+   STARTUPINFO st_info;
+   PROCESS_INFORMATION pr_info;
+   SECURITY_ATTRIBUTES attr;
+   HANDLE child_stdin, child_stdout, tmp;
+   char cmd[4096];
+
+   attr.nLength = sizeof(SECURITY_ATTRIBUTES);
+   attr.lpSecurityDescriptor = NULL;
+   attr.bInheritHandle = FALSE;
+
+   if (!CreatePipe(&tmp, &child_stdout, &attr, 0))
+      goto spice_error;
+
+   SetHandleInformation(tmp, HANDLE_FLAG_INHERIT, 0);
+   pipeRead = _open_osfhandle(tmp, _O_RDONLY);
+   if (!CreatePipe(&child_stdin, &tmp, &attr, 0))
+		goto spice_error;
+   SetHandleInformation(tmp, HANDLE_FLAG_INHERIT, 0);
+   pipeWrite = _open_osfhandle(tmp, _O_WRONLY);
+
+   ZeroMemory(&pr_info, sizeof(PROCESS_INFORMATION));
+   ZeroMemory(&st_info, sizeof(STARTUPINFO));
+   st_info.cb = sizeof(STARTUPINFO);
+   st_info.dwFlags = STARTF_USESTDHANDLES;
+   st_info.hStdError = st_info.hStdOutput = child_stdout;
+   st_info.hStdInput = child_stdin;
+
+   snprintf(cmd, 4095, "%s -p", SPICE_EXEC);
+   if (CreateProcess(NULL, cmd, NULL, NULL, TRUE, 0, NULL, NULL,
+		&st_info, &pr_info) == 0)
+      goto spice_error;
+
+   CloseHandle(pr_info.hThread);
+   spiceproc = pr_info.hProcess;
+   return 0;
+
+spice_error:
+   Fprintf(stderr, "Exec of ngspice failed\n");
+   return -2;
+}
+
+#else /* ! MSC_VER */
+
+int start_spice()
 {
    int std_in[2], std_out[2], ret;
 
@@ -125,7 +189,7 @@ void start_spice()
          execlp(SPICE_EXEC, "ngspice", "-p", (char *)NULL);
 	 spiceproc = -1;
 	 Fprintf(stderr, "Exec of ngspice failed\n");
-	 return;
+	 return -2;
       }
       else if (spiceproc < 0) {
 	 Wprintf("Error: ngspice not running");
@@ -133,17 +197,21 @@ void start_spice()
 	 close(std_in[1]);
 	 close(std_out[0]);
 	 close(std_out[1]);
-	 return;			/* error condition */
+	 return -1;			/* error condition */
       }
       else {	/* parent process */
 	 close(std_in[1]);
 	 close(std_out[0]);
 	 pipeRead = std_in[0];
 	 pipeWrite = std_out[1];
+	 return 0;			/* success */
       }
    }
+   return 1;	/* spice is already running */
 }
 
+#endif /* ! MSC_VER */
+
 /*------------------------------------------------------*/
 /* Send text to the ngspice simulator			*/
 /*------------------------------------------------------*/
@@ -284,13 +352,21 @@ int break_spice(Tcl_Interp *interp)
 {
    char *msg;
 
+#ifdef _MSC_VER
+   if (spiceproc == INVALID_HANDLE_VALUE) return 0;	 /* No process to interrupt */
+#else
    if (spiceproc == -1) return;	 /* No process to interrupt */
+#endif
 
    /* Sending SIGINT in any state other than "busy" will kill	*/
    /* the ngspice process, not interrupt it!			*/
 
    if (spice_state == SPICE_BUSY) {
+#ifndef _MSC_VER
       kill(spiceproc, SIGINT);
+#else
+      TerminateProcess(spiceproc, -1);
+#endif
       msg = recv_from_spice(interp, EXPECT_PROMPT);	/* Flush the output */
       if (*msg == '\0')
 	 return -1;
@@ -322,18 +398,30 @@ void resume_spice()
 
 int exit_spice()
 {
+#ifdef _MSC_VER
+   if (spiceproc == INVALID_HANDLE_VALUE) return -1;	/* ngspice not running */
+#else
    if (spiceproc < 0) return -1;	/* ngspice not running */
+#endif
 
 #ifdef SPICE_DEBUG
    fprintf(stdout, "Waiting for ngspice to exit\n");
 #endif
+#ifndef _MSC_VER
    kill(spiceproc, SIGKILL);
    waitpid(spiceproc, NULL, 0);      
+#else
+   TerminateProcess(spiceproc, -1);
+#endif
 #ifdef SPICE_DEBUG
    fprintf(stdout, "ngspice has exited\n");
 #endif
 
+#ifdef _MSC_VER
+   spiceproc = INVALID_HANDLE_VALUE;
+#else
    spiceproc = -1;
+#endif
    spice_state = SPICE_INIT;
 
    return 0;
@@ -378,10 +466,14 @@ int xctcl_spice(ClientData clientData, Tcl_Interp *interp,
    switch(idx) {
       case StartIdx:
 	 if (spice_state != SPICE_INIT) {
-	    Tcl_SetResult(interp, "spice process already running", NULL);
+	    Tcl_SetResult(interp, "ngspice process already running", NULL);
+	    return TCL_ERROR;
+	 }
+	 result = start_spice();
+	 if (result != 0) {
+	    Tcl_SetResult(interp, "unable to run ngspice", NULL);
 	    return TCL_ERROR;
 	 }
-	 start_spice();
 	 msg = recv_from_spice(interp, EXPECT_PROMPT);
 	 if (*msg == '\0')
 	    return TCL_ERROR;
