@@ -12,14 +12,18 @@
 #include <string.h>
 #include <math.h>
 
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#endif
 
 #ifdef TCL_WRAPPER 
 #include <tk.h>
 #else
+#ifndef XC_WIN32
 #include "Xw/Xw.h"
 #endif
+#endif
 
 /*-------------------------------------------------------------------------*/
 /* Local includes							   */
@@ -44,6 +48,37 @@ extern XCWindowData *areawin;
 extern char _STR[150];
 extern short textpos, textend;
 
+/*----------------------------------------------------------------------*/
+/* Prevent a list of elements from being selected.			*/
+/*----------------------------------------------------------------------*/
+
+void disable_selects(objectptr thisobject, short *selectlist, int selects)
+{
+   genericptr genptr;
+   short *i;
+
+   for (i = selectlist; i < selectlist + selects; i++) {
+      genptr = *(thisobject->plist + *i);
+      genptr->type |= SELECT_HIDE;
+   }
+}
+
+/*----------------------------------------------------------------------*/
+/* Allow a list of elements to be selected, if they were disabled using	*/
+/* the disable_selects() routine.					*/
+/*----------------------------------------------------------------------*/
+
+void enable_selects(objectptr thisobject, short *selectlist, int selects)
+{
+   genericptr genptr;
+   short *i;
+
+   for (i = selectlist; i < selectlist + selects; i++) {
+      genptr = *(thisobject->plist + *i);
+      genptr->type &= ~(SELECT_HIDE);
+   }
+}
+
 /*----------------------------------------------------------------------*/
 /* Change filter to determine what types can be selected		*/
 /*----------------------------------------------------------------------*/
@@ -59,7 +94,7 @@ void selectfilter(xcWidget w, pointertype value, caddr_t calldata)
       areawin->filter |= bitwise;
 
 #ifndef TCL_WRAPPER
-   toggle(w, &bval, calldata);
+   toggle(w, (pointertype)&bval, calldata);
 #endif
 }
 
@@ -717,6 +752,7 @@ void draw_all_selected()
 void draw_normal_selected(objectptr thisobj, objinstptr thisinst)
 {
    short *lastselect;
+   int i;
 
    if (areawin->selects == 0) return;
    else if (areawin->hierstack != NULL) return;
@@ -725,8 +761,8 @@ void draw_normal_selected(objectptr thisobj, objinstptr thisinst)
 
    SetFunction(dpy, areawin->gc, GXcopy);
 
-   for (lastselect = areawin->selectlist; lastselect < areawin->selectlist 
-	+ areawin->selects; lastselect++) {
+   for (i = 0; i < areawin->selects; i++) {
+      lastselect = areawin->selectlist + i;
       XTopSetForeground(SELTOCOLOR(lastselect)); 
       geneasydraw(*lastselect, SELTOCOLOR(lastselect), thisobj, thisinst);
    }
