@@ -24,6 +24,7 @@ extern void RegisterXPMImageType();
 
 extern int Tk_SimpleObjCmd();
 extern int xctcl_standardaction();
+extern int xctcl_action();
 extern int xctcl_refresh();
 extern int xctcl_library();
 extern int xctcl_font();
@@ -78,6 +79,7 @@ typedef struct {
 
 static cmdstruct xc_commands[] =
 {
+   {"action", (void *)xctcl_action},
    {"standardaction", (void *)xctcl_standardaction},
    {"start", (void *)xctcl_start},
    {"tag", (void *)xctcl_tag},
@@ -135,6 +137,9 @@ extern Tcl_HashTable XcTagTable;
 /* Initialization procedure for Tcl/Tk				*/
 /*--------------------------------------------------------------*/
 
+#ifdef _MSC_VER
+__declspec(dllexport)
+#endif
 int
 Xcircuit_Init(Tcl_Interp *interp)
 {
@@ -197,11 +202,11 @@ Xcircuit_Init(Tcl_Interp *interp)
 
    Tcl_SetVar(interp, "XCIRCUIT_LIB_DIR", tmp_s, TCL_GLOBAL_ONLY);
 
-   /* Set $CAD_HOME as a Tcl variable */
+   /* Set $CAD_ROOT as a Tcl variable */
 
-   cadhome = getenv("CAD_HOME");
+   cadhome = getenv("CAD_ROOT");
    if (cadhome == NULL) cadhome = CAD_DIR;
-   Tcl_SetVar(interp, "CAD_HOME", cadhome, TCL_GLOBAL_ONLY);
+   Tcl_SetVar(interp, "CAD_ROOT", cadhome, TCL_GLOBAL_ONLY);
 
    /* Set $XCIRCUIT_VERSION and $XCIRCUIT_REVISION as Tcl variables */
 
